comment ~
*****************************************************************************
         Example program for DOS32 dos-extender.


       Example on using the VESA BIOS Interface

Some of the vesa bios functions require a transfer of data in the
real mode address space. The mest way to do this is by taking advantage
of the 8Kb file buffer initally setup by DOS32. As explained in the API.DOC
this 8Kb buffer is in the real mode address space and may be use to temporary
hold information.

    If you are writting an program that uses VESA then to save time 
just copy and past the code from here into your asm files. As you can
see there is a fair bit of setting up reqired before you can start writting
to screen memory.

*****************************************************************************
~
.386
.MODEL FLAT , C
.STACK


VIDEO_MODE_NUMBER = 101h

; Data structure for VESA function 4F00h ( Get VBE Information )
;

     VbeInfoBlock struc
     VbeSignature        db   'VESA'    ; VBE Signature
     VbeVersion          dw   0200h     ; VBE Version
     OemStringPtr        dd   ?         ; Pointer to OEM String
     Capabilities        db   4 dup (?) ; Capabilities of graphics cont.
     VideoModePtr        dd   ?         ; Pointer to Video Mode List
     TotalMemory         dw   ?         ; Number of 64kb memory blocks
                                        ; Added for VBE 2.0
     OemSoftwareRev      dw   ?         ; VBE implementation Software revision
     OemVendorNamePtr    dd   ?         ; Pointer to Vendor Name String
     OemProductNamePtr   dd   ?         ; Pointer to Product Name String
     OemProductRevPtr    dd   ?         ; Pointer to Product Revision String
     _Reserved_          db   222 dup (?); Reserved for VBE implementation
                                        ; scratch area
     OemData             db   256 dup (?); Data Area for OEM Strings
     VbeInfoBlock ends



; Data structure for VESA function 4F01h ( Get mode Information )
;
     ModeInfoBlock   struc

     ; Mandatory information for all VBE revisions
     ModeAttributes      dw ?      ; mode attributes
     WinAAttributes      db ?      ; window A attributes
     WinBAttributes      db ?      ; window B attributes
     WinGranularity      dw ?      ; window granularity
     WinSize             dw ?      ; window size
     WinASegment         dw ?      ; window A start segment
     WinBSegment         dw ?      ; window B start segment
     WinFuncPtr          dd ?      ; pointer to window function
     BytesPerScanLine    dw ?      ; bytes per scan line

     ; Mandatory information for VBE 1.2 and above
     XResolution         dw ?      ; horizontal resolution in pixels or chars
     YResolution         dw ?      ; vertical resolution in pixels or chars
     XCharSize           db ?      ; character cell width in pixels
     YCharSize           db ?      ; character cell height in pixels
     NumberOfPlanes      db ?      ; number of memory planes
     BitsPerPixel        db ?      ; bits per pixel
     NumberOfBanks       db ?      ; number of banks
     MemoryModel         db ?      ; memory model type
     BankSize            db ?      ; bank size in KB
     NumberOfImagePages  db ?      ; number of images
     _Reserved           db ?      ; reserved for page function

     ; Direct Color fields (required for direct/6 and YUV/7 memory models)
     RedMaskSize         db ?      ; size of direct color red mask in bits
     RedFieldPosition    db ?      ; bit position of lsb of red mask
     GreenMaskSize       db ?      ; size of direct color green mask in bits
     GreenFieldPosition  db ?      ; bit position of lsb of green mask
     BlueMaskSize        db ?      ; size of direct color blue mask in bits
     BlueFieldPosition   db ?      ; bit position of lsb of blue mask
     RsvdMaskSize        db ?      ; size of direct color reserved mask in bits
     RsvdFieldPosition   db ?      ; bit position of lsb of reserved mask
     DirectColorModeInfo db ?      ; direct color mode attributes

     ; Mandatory information for VBE 2.0 and above
     PhysBasePtr         dd ?      ; physical address for flat frame buffer
     OffScreenMemOffset  dd ?      ; pointer to start of off screen memory
     OffScreenMemSize    dw ?      ; amount of off screen memory in 1k units
     __Reserved          db 206 dup (?)  ; remainder of ModeInfoBlock
     ModeInfoBlock ends

;
;  Macro to print some text on the screem
;
Print MACRO string
local @text,@skip
    mov edx,offset @text
    mov ah,9
    int 21h
    jmp @skip
@text db string,13,10,36
@skip:
ENDM


.DATA
file_buffer         DD ?
VideoPtr            DD ?
ProgramBaseAddress  DD ?
BankSetTable        DD 16 DUP (?)
cnt                 DB ?

DOS_segs LABEL DWORD
Real_ES     DW  ?
Real_DS     DW  ?

CurrentBank         DB  ?
.CODE


Palette  Label Byte
Include VESA.INC

;---------------------------------------
; SET Video bank number to AL
;---------------------------------------
SetBank PROC USES EDX EBX
        and    eax,0ffh
        mov    edx,BankSetTable[eax*4]
        mov    bh,0
        mov    bl,0
        mov    ax,04F05h
        push   10h
        call   DOSinterrupt
        Ret
SetBank ENDP


VBEstart PROC



         mov    ax,0EE02h       ; GET REAL MODE SEGMENT OF FILE I/O BUFFER
         int    31h

         ; returns AX = real mode segment of 8Kb buffer.
         ;         EBX = program base address.

         mov    ProgramBaseAddress,ebx
         mov    Real_ES,ax
         And    Eax,0FFFFh
         shl    Eax,4
         sub    eax,ebx
         Mov    file_buffer,eax     ; save near address of 8Kb file buffer.

     ;
     ;  GET VESA INFORMATION
     ;

         mov    ax,4F00h
         mov    di,0            ; (real mode) ES:DI -> 256 byte buffer
         push   10h
         call   DOSinterrupt
         Cmp    AX,004Fh
         jne   NoVESA


     ;
     ; Search video mode list for 640x480x256 ( VESA mode 101h )
     ;  The vesa driver Func 4F00h fills in the buffer with the information.
     ; Offset 0Eh of this buffer contains a real mode SEG:OFS of the video
     ; mode list. This list consists of each supported VESA mode and
     ; terminates with 0FFFFh.


       ; Get the real mode far pointer of the video mode list and
       ; convert the real mode SEG:OFS address into a 32bit near pointer.
       ;
         mov   eax,file_buffer
         Movzx edx,Word Ptr [ VbeInfoBlock.VideoModePtr + EAX + 2]
         shl   edx,4
         sub   edx,ProgramBaseAddress
         Movzx ebx,Word Ptr [ VbeInfoBlock.VideoModePtr + EAX ]
         add   edx,ebx                      ; EDX points to video mode list.
         xor eax,eax
Loop01:  Mov   ax,[Edx]                     ; Read video mode from list
         Cmp   Ax,0FFFFh
         je  @@ModeNotFound
         add   Edx,2
         cmp   Ax,VIDEO_MODE_NUMBER
         jne   Loop01


      ;
      ; Get VBE MODE information
      ; Note, the mode information block is also stored in the file buffer
      ;
        mov    ax,4F01h
        mov    cx,VIDEO_MODE_NUMBER
        mov    di,0             ; ES:DI -> 256 byte buffer
        push   10h
        call   DOSinterrupt
        Cmp    AX,004Fh
        jne   ModeNotGood



      ;
      ; Check if windowing is avalible ( writable and availible )
      ;
         Mov    Eax,file_buffer
         Test   [ ModeInfoBlock.ModeAttributes + EAX],01000000b
         jnz   ModeNotGood
         Mov    bl,[ ModeInfoBlock.WinAAttributes + EAX]
         and    bl,0000101b
         cmp    bl,0000101b
         jne   ModeNotGood


     ;
     ; Calulate a near pointer to CPU video memory Window A
     ;
         Mov    Eax,file_buffer
         Movzx  eax,[ ModeInfoBlock.WinASegment + EAX]
         shl    eax,4
         sub    eax,ProgramBaseAddress
         mov    VideoPtr,eax


    ;
    ; Setup Bank Numbering table
    ;
        Mov    Edi,file_buffer
        Movzx  eax,[ ModeInfoBlock.WinSize + EDI]
        Movzx  ebx,[ ModeInfoBlock.WinGranularity + EDI]
        xor    edx,edx
        div    ebx
        xor    edx,edx
        xor    ecx,ecx
@@loop01:
        mov    BankSetTable[ECX*4],edx
        add    edx,eax
        inc    ecx
        cmp    cl,LENGTH BankSetTable
        jb     @@loop01


     ;
     ; Set VIDEO mode  640x480x256 ..........
     ;
         mov    ax,4F02h
         mov    BX,VIDEO_MODE_NUMBER
         Int    10h
         Cmp    AX,004Fh
         jne    @@ModeNotFound

;----------------------------------------------------------------------------
; The video mode setting up is now complete and the applicaion is ready
; to start writting to video memory ( address defined in VideoPtr ).
;----------------------------------------------------------------------------




    ;
    ; Set palette
    ;
         mov    al,0
         mov    dx,3C8h
         out    dx,al
         mov    esi,Offset Palette
         mov    dx,3C9h
         mov    ecx,768
         cld
         rep    outsb


    ;
    ; Fill in screen with some patterns
    ;
         mov     CurrentBank,0
         mov     cnt,1
         mov     bl,0
loop02:  mov     edi,VideoPtr
         mov     al,CurrentBank
         call    SetBank
         mov     bh,65536/512
Loop03:  mov     ecx,512/4
         mov     al,bl
         mov     ah,bl
         push    ax
         push    ax
         pop     eax
         or al,0ffh
         rep     stosd
         add     bl,cnt
         jnz J98
         sub     bl,cnt
         neg     cnt
J98:     dec     bh
         jnz Loop03

         inc     CurrentBank
         cmp     CurrentBank,5
         jb loop02


        Print <'640 x 480 at 256 colors'>

        mov     ah,0        ; Wait for a key ( using BIOS service )
        int     16h

        mov    ax,3          ; Return to TEXT mode
        int    10h

Exit:
         Mov    Ax,4C00h    ; Exit this program
         Int    21h



;------------------------ Error messages ---------------------------------

ModeNotGood:
         Print <'VESA BIOS video mode 800x600x256 is incompatible'>
         jmp Exit
@@ModeNotFound:
         Print 'Video mode 640x480x256 not supported  ( VESA mode 101h )'
         jmp Exit

NoVESA:
         Print 'VESA BIOS not installed'
         jmp Exit

VBEstart ENDP




;------------------------------------------------------------
; Procudure to call a DOS interrupt.
;
; Expects the inerrupt number pushed on the stack.
;
;  e.g    push  10h
;         call  DOSinterrupt
;         jc   error
;
; As explained in the DOS32 documentaion, DOS32 initally handles all interrupts
; such that calling a protected mode interrupt will automatically call
; the same interrupt number in real mode.  However, using this method there
; is no way to specify the values of the real mode SEGMENT registers.
; Some of the VESA calls require ES:DI to point to buffers and so
; we need to use INT31h AX=300h service to call a real mode interrupt.
; The procedure below does exactly that...
;
;------------------------------------------------------------
DOSinterrupt PROC
        push    dword ptr 0             ; ignore  SS, SP
        lea     esp,[esp - 8]           ; ignore  CS, IP ,FS, GS
        push    [DOS_segs]              ; push DS and ES
        pushfw
        pushad
        mov     edi,esp
        mov     ax,0300h
        xor     cx,cx
        movzx   Ebx,Byte Ptr [esp+36h]  ; Get int number from stack param
        int     31h                     ; Emulate Real Mode Interrupt
        popad
        popfw
        lea     esp,[esp+16]            ; Ignore SS,SP,CS,IP,FS,GS,ES & DS
        ret     4                       ; return ingnoring parameter
DOSinterrupt ENDP

End VBEstart
